package com.mobisoftinfotech.rediscache.controller;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.CachePut;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.mobisoftinfotech.rediscache.model.Tutorial;
import com.mobisoftinfotech.rediscache.repository.TutorialRepository;

@RestController
@RequestMapping("/api")
public class TutorialController {

	@Autowired
	TutorialRepository tutorialRepository;

	@GetMapping("/tutorials")
	public ResponseEntity<List<Tutorial>> getAllTutorials() {
		List<Tutorial> tutorials = new ArrayList<>();
		tutorialRepository.findAll().forEach(tutorials::add);
		return new ResponseEntity<>(tutorials, HttpStatus.OK);
	}

	@GetMapping("/tutorials/{id}")
	@Cacheable(value = "tutorials", key = "#id")
	public Tutorial getTutorialById(@PathVariable("id") long id) {

		Optional<Tutorial> tutorialData = tutorialRepository.findById(id);
		if (tutorialData.isPresent()) {
			return tutorialData.get();
		}
		return null;
	}

	@PostMapping("/tutorials")
	public ResponseEntity<Tutorial> createTutorial(@RequestBody Tutorial dto) {
		Tutorial tutorial = tutorialRepository.save(new Tutorial(dto.getTitle(), dto.getDescription(), false));
		return new ResponseEntity<>(tutorial, HttpStatus.CREATED);
	}

	@PutMapping("/tutorials/{id}")
	@CachePut(value = "tutorials", key = "#id")
	public Tutorial updateTutorial(@PathVariable("id") long id, @RequestBody Tutorial tutorial) {
		Optional<Tutorial> tutorialData = tutorialRepository.findById(id);

		if (tutorialData.isPresent()) {
			Tutorial tutorialResp = tutorialData.get();
			tutorialResp.setTitle(tutorial.getTitle());
			tutorialResp.setDescription(tutorial.getDescription());
			tutorialResp.setPublished(tutorial.isPublished());
			tutorialRepository.save(tutorialResp);
			return tutorialResp;
		}
		return null;
	}

	@DeleteMapping("/tutorials/{id}")
	@CacheEvict(value = "tutorials", key = "#id")
	public ResponseEntity<HttpStatus> deleteTutorial(@PathVariable("id") long id) {
		tutorialRepository.deleteById(id);
		return new ResponseEntity<>(HttpStatus.NO_CONTENT);
	}

}
